Package: statmod
Version: 1.5.2
Date: 2026-05-17
Title: Statistical Modeling
Authors@R: c(person(given = "Gordon",
                    family = "Smyth",
                    role = c("cre", "aut"),
                    email = "smyth@wehi.edu.au"),
             person(given = "Lizhong",
                    family = "Chen",
                    role = "aut"),
             person(given = "Yifang",
                    family = "Hu",
                    role = "ctb"),
             person(given = "Peter",
                    family = "Dunn",
                    role = "ctb"),
             person(given = "Belinda",
                    family = "Phipson",
                    role = "ctb"),
             person(given = "Yunshun",
                    family = "Chen",
                    role = "ctb"))
Maintainer: Gordon Smyth <smyth@wehi.edu.au>
Depends: R (>= 3.0.0)
Imports: stats, graphics
Suggests: MASS, tweedie
Description: A collection of algorithms and functions to aid statistical modeling. Includes limiting dilution analysis (aka ELDA), growth curve comparisons, mixed linear models, heteroscedastic regression, inverse-Gaussian probability calculations, Gauss quadrature and a secure convergence algorithm for nonlinear models. Also includes advanced generalized linear model functions including Tweedie and Digamma distributional families, secure convergence and exact distributional calculations for unit deviances.
License: GPL-2 | GPL-3
NeedsCompilation: yes
Packaged: 2026-05-17 10:47:09 UTC; smyth
Author: Gordon Smyth [cre, aut],
  Lizhong Chen [aut],
  Yifang Hu [ctb],
  Peter Dunn [ctb],
  Belinda Phipson [ctb],
  Yunshun Chen [ctb]
Repository: CRAN
Date/Publication: 2026-05-17 15:10:02 UTC
